//
//  命中&回避計算式 ver1.01
//
// ------------------------------------------------------
// Copyright (c) 2016 Yana
// Released under the MIT license
// http://opensource.org/licenses/mit-license.php
// ------------------------------------------------------
//
// author Yana
//

var Imported = Imported || {};
Imported['HitandEvaFormula'] = 1.01;
/*:
 * @plugindesc ver1.01/命中や回避に計算式が使用できるようにします。
 * @author Yana
 * 
 * @param HitFormula
 * @desc 命中計算の基本式です。
 * @default a.agi - b.luk
 * 
 * @param EvaFormula
 * @desc 回避計算の基本式です。
 * @default 0
 *
 * @param HitCountMessage
 * @desc 攻撃の命中回数のメッセージです。
 * _cntを命中数、_nameを名前に変更します。
 * @default _cnt回ヒット！
 * 
 * @help------------------------------------------------------
 * 使用方法
 * ------------------------------------------------------
 * アイテムやスキルのメモに、
 * <命中計算式:xxx>
 * または、
 * <HitFormula:xxx>
 * と記述すると、命中の計算式がxxxに設定されます。
 * xxxでは、通常のダメージ計算式で使用できるa,b,vに加え、
 * hit(行動者の命中値),eva(対象の回避値-魔法なら魔法回避値)が使用可能です。
 * 
 * アイテムやスキルのメモに
 * <回避計算式:xxx>
 * または、
 * <EvaFormula:xxx>
 * と記述すると、回避の計算式がxxxに設定されます。
 * xxxでは、通常のダメージ計算式で使用できるa,b,vに加え、
 * hit(行動者の命中値),eva(対象の回避値(魔法なら魔法回避値))が使用可能です。
 * 
 * 個別で式が設定されていないアイテムやスキルは、パラメータで使用された式が設定されます。
 *
 * ver1.01より、複数回判定の仕組みを追加しました。
 * スキルやアイテムのメモに
 * <判定回数:○>
 * <JudgeCount:○>
 * のいずれかを記述すると、命中判定時に回数分の判定を行います。
 * そのいずれかで命中すれば命中となり、命中した分をすべて回避すれば回避となります。
 * 命中回数にはb.hc()でアクセス可能です。
 * 
 * ------------------------------------------------------
 * 利用規約
 * ------------------------------------------------------
 * 当プラグインはMITライセンスで公開されています。
 * 使用に制限はありません。商用、アダルト、いずれにも使用できます。
 * 二次配布も制限はしませんが、サポートは行いません。
 * 著作表示は任意です。行わなくても利用できます。
 * 要するに、特に規約はありません。
 * バグ報告や使用方法等のお問合せはネ実ツクールスレ、または、Twitterにお願いします。
 * https://twitter.com/yanatsuki_
 * 素材利用は自己責任でお願いします。
 * ------------------------------------------------------
 * 更新履歴:
 * ver1.01:
 * 利用規約をMITライセンスに変更。
 * 判定回数の仕組みを追加。
 * ver1.00:
 * 公開
 */

(function(){
	////////////////////////////////////////////////////////////////////////////////////
	
	var parameters = PluginManager.parameters('HitandEvaFormula');
	var hitFormula = String(parameters['HitFormula']);
	var evaFormula = String(parameters['EvaFormula']);
	var hitCountMessage = String(parameters['HitCountMessage']) || '';
	
	////////////////////////////////////////////////////////////////////////////////////

	DataManager.hitFormula = function(item) {
		if (item._hitFormula === undefined){
			item._hitFormula = item.meta['命中計算式'];
			if (!item._hitFormula){ item._hitFormula = item.meta['HitFormula'] }	
			if (!item._hitFormula){ item._hitFormula = hitFormula }
		}
		return item._hitFormula;
	};

	DataManager.evaFormula = function(item) {
		if (item._evaFormula === undefined){
			item._evaFormula = item.meta['回避計算式'];
			if (!item._evaFormula){ item._evaFormula = item.meta['EvaFormula'] }
			if (!item._evaFormula){ item._evaFormula = evaFormula }
		}
		return item._evaFormula;
	};

	////////////////////////////////////////////////////////////////////////////////////

	Game_Battler.prototype.hc = function() {
		return this.result()._hitCount;
	};

	////////////////////////////////////////////////////////////////////////////////////
	
	var __GAction_itemHit = Game_Action.prototype.itemHit;
	Game_Action.prototype.itemHit = function(target) {
		console.log('itemHit');

		target.result()._hitCount = 0;
		var hit = __GAction_itemHit.call(this,target);
		var eva = __GAction_itemEva.call(this,target);
    	var a = this.subject();
    	var b = target;
    	var v = $gameVariables._data;
        var s = $gameSwitches._data;
        var n = eval(this.item().meta['判定回数'] || this.item().meta['JudgeCount'] || 1);
        var result = eval(DataManager.hitFormula(this.item()));
		var randomValue = 0;

		//必中なら必ずヒット
		if($dataSkills[this._item._itemId].hitType == 0)
		{
			console.log('必中');
			result = 1.0;
		}
		else
		{
	        for (var i = 0; i < n; i++) {
				//rkrk
				randomValue = Math.floor(Math.random() * 11 - 5)
				JudgeValue = result + randomValue;

				//
				if(this.item._hitType != 0)
				{
					var message = '命中力';
					if(JudgeValue < 0)
					{
						message += (JudgeValue*-1)+'不足';
					}
					else if(JudgeValue > 0)
					{
						message += 'が回避力を' + JudgeValue + '上回った';
					}
					else
					{
						message += 'が回避力と同値';
					}
					if(BattleManager)
					{
						BattleManager._logWindow.addText(message);
					}
				}

	            if (JudgeValue >= 0) {target.result()._hitCount++;}
				this.subject()._meichuuDoai=JudgeValue;
				console.log(this.subject());
	        }
	        result = target.result()._hitCount > 0 ? 1.0 : 0;
		}
		
		//rkrk
		console.log('result');
		console.log(result);
    	return result;
	};

	var __GAction_itemEva = Game_Action.prototype.itemEva;
	Game_Action.prototype.itemEva = function(target) {
	    if (target.result()._hitCount === undefined) target.result()._hitCount = 1;
		var eva = __GAction_itemEva.call(this,target);
    	var hit = __GAction_itemHit.call(this,target);
    	var a = this.subject();
    	var b = target;
    	var v = $gameVariables._data;
        var s = $gameSwitches._data;
        var result = eval(DataManager.evaFormula(this.item()));
        for (var i=0,max=target.result()._hitCount;i<max;i++) {
            if (result > Math.random()) target.result()._hitCount--;
        }
        result = target.result()._hitCount > 0 ? 0 : 1.0;
    	//result = Math.min(1.0,Math.max(result,0));
    	//return result;
		return 0;
	};

	////////////////////////////////////////////////////////////////////////////////////

    Window_BattleLog.prototype.displayHitCount = function(target) {
        if (target.result()._hitCount && hitCountMessage && target.result()._hitCount > 1) {
            var message = hitCountMessage.replace(/_cnt/,target.result()._hitCount);
            message = message.replace(/_name/,target.name());
            //this.addText(message);
            this.push('addTextNoWait', message);
        }
    };

	var __WBattleLog_displayDamage = Window_BattleLog.prototype.displayDamage;
	Window_BattleLog.prototype.displayDamage = function(target) {
        this.displayHitCount(target);
		__WBattleLog_displayDamage.call(this, target);
	};

    Window_BattleLog.prototype.addTextNoWait = function(text) {
        this._lines.push(text);
        this.refresh();
    };

	////////////////////////////////////////////////////////////////////////////////////

Window_BattleLog.prototype.displayActionResults = function(subject, target) {
	console.log('displayActionResults');
	//console.log(target.result());
    if (target.result().used) {
        this.push('pushBaseLine');

        this.displayCritical(target);
        this.push('popupDamage', target);
        this.push('popupDamage', subject);
        this.displayDamage(target);

        this.displayAffectedStatus(target);

       	this.displayFailure(target);

        this.push('waitForNewLine');
        this.push('popBaseLine');
    }
};

BattleManager.invokeAction = function(subject, target) {
    this._logWindow.push('pushBaseLine');
    if (Math.random() < this._action.itemCnt(target)) {
        this.invokeCounterAttack(subject, target);
    } else if (Math.random() < this._action.itemMrf(target)) {
        this.invokeMagicReflection(subject, target);
    } else {
        this.invokeNormalAction(subject, target);
    }

	//rkrk
	//subject._lastAction=this._action;

    subject.setLastTarget(target);
    this._logWindow.push('popBaseLine');
    this.refreshStatus();
};

Window_BattleLog.prototype.displayAction = function(subject, item) {
console.log('displayAction');
//console.log(item);
    var numMethods = this._methods.length;
    if (DataManager.isSkill(item)) {
        if (item.message1) {
            this.push('addText', subject.name() + item.message1.format(item.name));
        }
        if (item.message2) {
            this.push('addText', item.message2.format(item.name));
        }

		/*
		//rkrk
		if(item.hitType != 0)
		{
			var message = '命中力';
	
			//rkrk

			console.log('命中度合い');
			console.log(subject._meichuuDoai);

			if(subject._meichuuDoai < 0)
			{
				message += (subject._meichuuDoai * -1) + '不足';
			}
			else if(subject._meichuuDoai > 0)
			{
				message += 'が回避力を' + subject._meichuuDoai + '上回った';
			}
			else
			{
				message += 'が回避力と同値';
			}
			this.push('addText',message);
		}
		*/

    } else {
        this.push('addText', TextManager.useItem.format(subject.name(), item.name));
    }
    if (this._methods.length === numMethods) {
        this.push('wait');
    }
};

Game_Action.prototype.apply = function(target) {
    var result = target.result();
    this.subject().clearResult();
    result.clear();
    result.used = this.testApply(target);
    result.missed = (result.used && Math.random() >= this.itemHit(target));
    result.evaded = (!result.missed && Math.random() < this.itemEva(target));
    result.physical = this.isPhysical();
    result.drain = this.isDrain();
    if (result.isHit()) {
        if (this.item().damage.type > 0) {
            result.critical = (Math.random() < this.itemCri(target));
            var value = this.makeDamageValue(target, result.critical);
            this.executeDamage(target, value);
        }
        this.item().effects.forEach(function(effect) {
            this.applyItemEffect(target, effect);
        }, this);
        this.applyItemUserEffect(target);
    }
};


}());